<?php defined('SYSPATH') or die('No direct script access.');

class FileParser
{
    /**
     * Parse students' info & synchronize it with database.
     * @param $filename string File with source data
     * @param $facultyID int
     * @return array errors
     */
    public static function uploadStudents($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");

        $i = 0;
        $errors = [];

        while ($line = fgetcsv($file, 0, ";")) {
            // ФИО, курс, степень подготовки,
            list($name, $gradeNum, $groupNum, $degree, $spec) = $line;

            // Фамилия, имя, отчество
            list($lastName, $firstName, $secondName) = self::parsePeopleName($name);

            $degree = Model_Grades::getDegreeType($degree);

            $attempt = Model_Account::createStudentEx(
                $lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $spec, $facultyID
            );

            if ($attempt == -1) {
                $errors[] = ['Row' => $i, 'Info' => implode(';', $line)];
            }

            $i++;
        }

        return $errors;
    }

    /**
     * Parse subjects' info & synchronize it with database.
     * @param $filename  string File with source data
     * @param $facultyID int
     * @return array
     */
    public static function uploadSubjects($filename, $facultyID) {
        if (File::mime($filename) != 'text/plain')
            return [];

        $file = fopen($filename, "r");
        $count = $exists = 0;
        $errors = [];

        while (list($name, $abbr) = fgetcsv($file, 0, ";")) {
            $attempt = Model_Subject::create($name, $abbr, $facultyID);

            if ($attempt < 0) {
                $errors[] = ['Row' => $count, 'Info' => $name . ';' . $abbr];
            } elseif ($attempt == 1) {
                $exists++;
            }

            $count++;
        }

        return [
            'Errors'             => $errors,
            'ErrorsCount'        => count($errors),
            'RecordsCount'       => $count,
            'RecordsExistsCount' => $exists,
        ];
    }

    public static function TeachersList($filename)
    {
        // get faculties list
        $dbFaculties = Model_Faculties::load();
        foreach ($dbFaculties as $rec) {
            $name = trim($rec['Name']);
            $id = $rec['ID'];
            $faculties[$name] = $id;
        }

        if(File::mime($filename) != 'text/plain')
            return true;
        $file = fopen($filename, "r");
        $i = $j = 0;
        while ($line = fgetcsv($file, 0, ";"))
        {
            // Имя, фамилия, отчество
            list($lastName, $firstName, $secondName) = self::parsePeopleName($line[0]);
            $departmentName = $line[1];
            $facultyID = 6;  // TODO fix facultyID hardcode
            if ($line[2])
                $facultyID = $faculties[trim($line[2])];
            $attempt = Account::instance()
                ->createTeacherByDepName($lastName, $firstName, $secondName, $departmentName, $facultyID);

            if($attempt == -1)
            {
                $resultArr[$j]['Row'] = $i;
                $resultArr[$j]['Info'] = implode(';', $line);
                $j++;
            }
            $i++;
        }
        return ($j > 0) ? false : $resultArr;
    }

    private static function parsePeopleName($name) {
        $nameExploded = explode(' ', $name);

        $idx = 0;
        $nameHandled = [];

        // Енотова (Сенченко) Наталья Лин Чу Геннадьевна
        if (UTF8::substr($nameExploded[1], 0, 1) == '(') {
            $idx = 1;
        }

        $nameHandled[0] = $nameExploded[0];
        $nameHandled[1] = $nameExploded[1 + $idx];
        $nameHandled[2] = '';

        for ($i = 2 + $idx; $i < count($nameExploded); $i++) {
            $nameHandled[2] .= $nameExploded[$i] . ' ';
        }

        return Arr::map('trim', $nameHandled);
    }
}